@@ -13,7 +13,7 @@ from tornado.ioloop import IOLoop
 from tornado.options import options
 from webssh.utils import (
     is_valid_ip_address, is_valid_port, is_valid_hostname, to_bytes, to_str,
-    to_int, to_ip_address, UnicodeType, is_name_open_to_public, is_ip_hostname
+    to_int, to_ip_address, UnicodeType, is_ip_hostname, is_same_primary_domain
 )
 from webssh.worker import Worker, recycle_worker, clients
 
@@ -27,24 +27,18 @@ try:
 except ImportError:
     JSONDecodeError = ValueError
 
+try:
+    from urllib.parse import urlparse
+except ImportError:
+    from urlparse import urlparse
+
 
 DELAY = 3
 KEY_MAX_SIZE = 16384
 DEFAULT_PORT = 22
 
 swallow_http_errors = True
-
-# set by config_open_to_public
-open_to_public = {
-    'http': None,
-    'https': None
-}
-
-
-def config_open_to_public(address, server_type):
-    status = True if is_name_open_to_public(address) else False
-    logging.debug('{} server open to public: {}'.format(server_type, status))
-    open_to_public[server_type] = status
+redirecting = None
 
 
 class InvalidValueError(Exception):
@@ -63,6 +57,7 @@ class MixinHandler(object):
     def initialize(self, loop=None):
         self.check_request()
         self.loop = loop
+        self.origin_policy = self.settings.get('origin_policy')
 
     def check_request(self):
         context = self.request.connection.context
@@ -81,9 +76,32 @@ class MixinHandler(object):
         else:
             self.context = context
 
+    def check_origin(self, origin):
+        if self.origin_policy == '*':
+            return True
+
+        parsed_origin = urlparse(origin)
+        netloc = parsed_origin.netloc.lower()
+        logging.debug('netloc: {}'.format(netloc))
+
+        host = self.request.headers.get('Host')
+        logging.debug('host: {}'.format(host))
+
+        if netloc == host:
+            return True
+
+        if self.origin_policy == 'same':
+            return False
+        elif self.origin_policy == 'primary':
+            return is_same_primary_domain(netloc.rsplit(':', 1)[0],
+                                          host.rsplit(':', 1)[0])
+        else:
+            return origin in self.origin_policy
+
     def is_forbidden(self, context, hostname):
         ip = context.address[0]
         lst = context.trusted_downstream
+        ip_address = None
 
         if lst and ip not in lst:
             logging.warning(
@@ -91,13 +109,17 @@ class MixinHandler(object):
             )
             return True
 
-        if open_to_public['http'] and context._orig_protocol == 'http':
-            if not to_ip_address(ip).is_private:
-                if open_to_public['https'] and options.redirect:
-                    if not is_ip_hostname(hostname):
-                        # redirecting
-                        return False
-                if options.fbidhttp:
+        if context._orig_protocol == 'http':
+            if redirecting and not is_ip_hostname(hostname):
+                ip_address = to_ip_address(ip)
+                if not ip_address.is_private:
+                    # redirecting
+                    return False
+
+            if options.fbidhttp:
+                if ip_address is None:
+                    ip_address = to_ip_address(ip)
+                if not ip_address.is_private:
                     logging.warning('Public plain http request is forbidden.')
                     return True
 
@@ -234,7 +256,7 @@ class IndexHandler(MixinHandler, tornado.web.RequestHandler):
 
     def get_hostname(self):
         value = self.get_value('hostname')
-        if not (is_valid_hostname(value) | is_valid_ip_address(value)):
+        if not (is_valid_hostname(value) or is_valid_ip_address(value)):
             raise InvalidValueError('Invalid hostname: {}'.format(value))
         return value
 
@@ -340,6 +362,13 @@ class IndexHandler(MixinHandler, tornado.web.RequestHandler):
         if len(clients.get(self.src_addr[0], {})) >= options.maxconn:
             raise tornado.web.HTTPError(403, 'Too many connections.')
 
+        origin = self.get_argument('_origin', u'')
+        if origin:
+            if not self.check_origin(origin):
+                raise tornado.web.HTTPError(
+                        403, 'Cross origin frame operation is not allowed.'
+                    )
+
         future = Future()
         t = threading.Thread(target=self.ssh_connect_wrapped, args=(future,))
         t.setDaemon(True)
